define close_match (a, b)
%!%+
%!%\function{close_match}
%!%\synopsis{Return index array for the larger array where
%!%smaller array has closest matching value.}
%!%\usage{Int_Type[] n = close_match(Array_Type a, Array_Type b);
%!%}
%!%\description
%!%    Takes two numerical arrays and returns the index array such that
%!%    the larger array is approximated with the smaller one by
%!%    \code{a[n]} or \code{b[n]}.
%!%    
%!%\note
%!%    If the arrays have only equal values, then a direct comparison
%!%    is much faster. close_match is allwas O(N^2).
%!%-
{
  variable k,j;
  variable idxo;
  variable lo,li,o,i;
  if (length(a)<length(b)) {
    lo = length(b);
    li = length(a);
    o = &b;
    i = &a;
  } else {
    lo = length(a);
    li = length(b);
    o = &a;
    i = &b;
  }
  idxo = @Int_Type[lo];
  %idxi = @Int_Type[li];

  variable diff, mindiff=_Inf;
  variable minidxi;
  _for k (0, lo-1, 1) {
    _for j (0, li-1, 1) {
      diff = abs((@o)[k]-(@i)[j]);
      if (diff < mindiff) {
	minidxi = j;
	mindiff = diff;
      }
    }
    idxo[k] = minidxi;
    mindiff = _Inf;
  }
  return idxo;
}