define eval_array()
%!%+
%\function{eval_array}
%\synopsis{evaluate an expression for an array of values}
%\usage{Array_Type eval_array(Type_Type type, String_Type expression, Array_Type values)
%\altusage{eval_array(String_Type expression, Array_Type values);}
%}
%\qualifiers{
%\qualifier{marker}{[\code{="*"}] place holder in the \code{expression} string,
%                     where the \code{values} are to be inserted}
%\qualifier{n_markers}{[=all] number of markers in \code{expression} to be replaced;
%                        if positive, the first \code{n_markers} are replaced,
%                        if negative, the last \code{|n_markers|} are replaced.}
%}
%\description
%    The \code{marker} string in \code{expression} is sequentially replaced with
%    each value of the array \code{values} (note that the values are converted
%    through the \code{string} function), and the resulting string is evaluated.
%    Unless \code{type==Void_Type}, each evaluation is expected to result in one value
%    of this \code{type}, and \code{eval_array} returns an array of all these values.
%    If \code{type==Void_Type} (which can be omitted), no return value is expected.
%\examples
%    \code{%} set fields in an array of structures\n
%    \code{public variable s = Struct_Type[8];  _for $1 (0, 7, 1)  s[$1] = struct { a, b };}\n
%    \code{eval_array("s[*].a = 1", [0:3]);  % =>  s[[0:3]].a = 1;}\n
%    \code{eval_array("s[*].a = 2", [4:7]);  % =>  s[[4:7]].a = 2;}\n
%
%    \code{%} using the array several times\n
%    \code{eval_array("s[*].b = *",     [0:3]);               % =>  s[[0:3]].b =  [0:3];}\n
%    \code{eval_array("s[*].b = * * 2", [4:7]; n_markers=2);  % =>  s[[4:7]].b =  [4:7] * 2;}\n
%
%    \code{%} using a different marker to clarify the last example\n
%    \code{eval_array("s[#].b = 2 * #", [4:7]; marker="#");   % =>  s[[4:7]].b =  2 * [4:7];}\n
%\seealso{array_map, array_struct_field, eval}
%!%-
{
  variable type=Void_Type, expression, values;
  switch(_NARGS)
  { case 2: (expression, values) = (); }
  { case 3: (type, expression, values) = (); }
  { help(_function_name()); return; }

  variable i, n = length(values), value;
  variable marker = qualifier("marker", "*");
  variable n_markers = qualifier("n_markers", strlen(expression));
  if(type != Void_Type)
  {
    variable a = type[n];
    _for i (0, n-1, 1)
    {
      strreplace(expression, marker, string(values[i]), n_markers);  % (replaced_string, n_replacements) left on stack
      _pop_n(1);                                                     % remove n_replacements from stack
      a[i] = eval;                                                   % evaluate replaced_string
    }
    return a;
  }
  else
    foreach value (values)
    {
      strreplace(expression, marker, string(value), n_markers);  % (replaced_string, n_replacements) left on stack
      _pop_n(1);                                                 % remove n_replacements from stack
      eval;                                                      % evaluate replaced_string
    }
}
