define matrix2array()
%!%+
%\function{matrix2array}
%\synopsis{Transforms a matrix to an Array}
%\usage{Array_Type[] matrix2array( Any_Type[...,d,...] M, d );}
%\description
%     From the given dimension d of a matrix M an array is created,
%     i.e., the given dimension of the matrix is shifted into the
%     array which then contains matrices without this dimension.
%\example
%     variable M = _reshape( [1:2*3*4], [2,3,4] );
%     variable A0 = matrix2array(M,0);
%     variable A2 = matrix2array(M,2);
%     vmessage("A0");print(A0);
%     vmessage("A2");print(A2);
%!%-
{
  variable M, d;
  switch(_NARGS)
  { case 2: ( M, d ) = (); }
  { help(_function_name); return; }

  variable dim = array_shape(M);
  variable dlen= length(dim);
  
  variable I1 = Array_Type[d==0?0:d];
  I1[*] = [*];
  variable I2 = Array_Type[d==dlen-1?0:dlen-d-1];
  I2[*] = [*];
  
  variable A = Array_Type[dim[d]];
  _for $1 ( 0, dim[d]-1 ){
    A[$1] = M[__push_array(I1), $1, __push_array(I2) ];
  }

  return A;
}
