%%%%%%%%%%%%%%%%%%%%
define outer_product()
%%%%%%%%%%%%%%%%%%%%
%!%+
%\function{outer_product}
%\synopsis{computes the outer product of a number of vectors}
%\usage{Double_Type P[] = outer_product(Double_Type f1[], f2[], ..., fn[]);}
%\description
%    The \code{n} arguments \code{f1}, \code{f2}, ..., \code{fn} have to be one-dimensional arrays.
%    The return value \code{P} is an \code{n}-dimensional array with
%    \code{P[i1, i2, ..., in] = f1[i1] * f2[i2] * ... fn[in]}.
%!%-
{
  if(_NARGS==0)
  { help(_function_name()); return; }

  variable n = _NARGS;
  variable arg = __pop_args(n);
  arg = array_map( Array_Type, &get_struct_field, arg, "value" );
  variable dim = array_map( Integer_Type, &length, arg );

  variable P = Double_Type[nint(prod(dim))];
  P[[0:dim[0]-1]] = arg[0];
  
  variable I, len, _len = dim[0];
  variable ii;
  _for ii ( 1, length(dim)-1 ){
    len = nint(prod(dim[[0:ii]]));
    I = [0:len-1];
    P[I] = P[I/dim[ii] mod _len] * arg[ii][I mod dim[ii]];
    _len = @len;
  }
  
  reshape(P, dim);
  return P;
}

% %%%%%%%%%%%%%%%%%%%%
% define outer_product_old()
% %%%%%%%%%%%%%%%%%%%%
% %!%+
% %\function{outer_product}
% %\synopsis{computes the outer product of a number of vectors}
% %\usage{Double_Type P[] = outer_product(Double_Type f1[], f2[], ..., fn[]);}
% %\description
% %    The \code{n} arguments \code{f1}, \code{f2}, ..., \code{fn} have to be one-dimensional arrays.
% %    The return value \code{P} is an \code{n}-dimensional array with
% %    \code{P[i1, i2, ..., in] = f1[i1] * f2[i2] * ... fn[in]}.
% %!%-
% {
%   if(_NARGS==0)
%   { help(_function_name()); return; }

%   variable n = _NARGS;
%   variable arg = __pop_args(n);
%   variable dim = Integer_Type[n];
%   variable d, total_length = 1;
%   _for d (0, n-1, 1)
%   { dim[d] = array_shape(arg[d].value)[0];
%     total_length *= dim[d];
%   }
%   variable P = Double_Type[total_length];
%   variable i = Integer_Type[n];
%   variable j;
%   _for j (0, total_length-1, 1)
%   { $1 = 1;
%     _for d (0, n-1, 1)
%       $1 *= arg[d].value[i[d]];
%     P[j] = $1;
%     _for d (n-1, 0, -1)
%     { if(i[d]<dim[d]-1)
%       {	i[d]++;
% 	break;
%       }
%       else
%         i[d] = 0;
%     }
%   }
%   reshape(P, dim);
%   return P;
% }
