define print_array(arr)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% %{{{
%!%+
%\function{print_array}
%\synopsis{Prints the entries of an array in one line.}
%\usage{print_array(Array_Type);}
%\description
%     Prints the entries of an array in one line instead of different lines.
%     Helpful for displaying dependent values of different arrays like a table.
%\example
%     UVOT-filter names and their wavelengths.
%
%     isis> filters=["UVW2","UVM2","UVW1","U","B","V"];
%     isis> wave=[2078.546871,2257.186625,2659.009185,3475.482080,4359.034280,5429.548507];
%     isis> print_array(filters); print_array(wave);
%     UVW2    UVM2    UVW1    U       B       V
%     2078.546871     2257.186625     2659.009185     3475.482080     4359.034280     5429.548507
%!%-   
{
    variable ty; switch(_typeof(arr))
     {                                                                                                                 
         case
          Integer_Type: ty = "%d";
     }                            
     {                            
      case Double_Type: if (min(arr) < 1e-4) ty ="%e";
          else ty = "%f";                             
     }                                                
     {                                                
         case                                         
          String_Type: ty = "%s";                     
     }                                                
   ; ()=array_map(Integer_Type, &printf,              
                  sprintf("%s\t", ty), arr); message("");
} %}}}                                                       
