define Roche_equipotential_volume()
%!%+
%\function{Roche_equipotential_volume}
%\synopsis{computes the volume inside an Roche equipotential surface}
%\usage{Double_Type Roche_equipotential_volume}
%\qualifiers{
%\qualifier{phi}{[= critical potential] the surface's Roche potential}
%\qualifier{N}{[= 100] number of subdivisions in x and y for numerical integration}
%}
%!%-
{ variable q;
  switch(_NARGS)
  { case 1: q = (); }
  { help(_function_name()); return; }

  variable RocheLobe = qualifier("RocheLobe", NULL);
  if(RocheLobe==NULL)  RocheLobe = Roche_lobe_dimension(q);
  variable phi = qualifier("phi", RocheLobe.potential);
  variable N = qualifier("N", 100);
  variable x, X = [RocheLobe.xmin:RocheLobe.xmax:#N];
  variable y, Y = [0:RocheLobe.ymax:#N];
  variable V = 0;
  variable wx = 1;
  foreach x (X)
  { if(x==X[-1])  wx = 1;
    variable wy = 1;
    foreach y (Y)
    { if(y==Y[-1])  wy = 1;
      if(x!=0 || y!=0)
      { if(Roche_potential(x, y, 0; q=q) < phi)
          V += wx*wy*find_multiargumentfunction_value(&Roche_potential, phi, x, y, [0, 1.01*RocheLobe.zmax]; qualifiers=struct{q=q});
      }
      else
        V += wx*wy*find_multiargumentfunction_value(&Roche_potential, phi, x, y, [0.99, 1.01]*RocheLobe.zmax; qualifiers=struct{q=q});
      if(wy==1||wy==2)  wy = 4;  else  wy = 2;
    }
    if(wx==1||wx==2)  wx = 4;  else  wx = 2;
  }
  return 4*V/9.*(X[1]-X[0])*(Y[1]-Y[0]);  % 4 due to symmetry in y and z, 9 is from the 2D Simpson rule
}
