define complex2rgb()
%!%+
%\function{complex2rgb}
%\synopsis{converts a complex number to a RGB color}
%\usage{Integer_Type complex2rgb(Complex_Type z);}
%\description
%    arg(code{z}) determines the hue,
%    |\code{z}| determines the lightness:
%    0<=|\code{z}|<=1 is mapped from black to color,
%    1<=|\code{z}|<inf is mapped from color to white.
%    The return value is a 24-bit RGB value.
%\qualifiers{
%\qualifier{sat}{[=1]: saturation (from 0 to 1) of the color}
%}
%\seealso{hsl2rgb, png_write}
%!%-
{
  variable z;
  switch(_NARGS)
  { case 1: z = (); }
  { help(_function_name()); return; }

  variable arrayshape = NULL;
  variable len = length(z);
  if(typeof(z)==Array_Type)
  { arrayshape = array_shape(z);
    reshape(z, len);
  }
  variable RGB = Integer_Type[len];

  variable s = _max(0, _min(1, qualifier("sat", 1)));

  variable i;
  _for i (0, len-1, 1)
  { variable z0 = [z][i];
    variable h = (atan2(Imag(z0), Real(z0))/(2*PI) + 1.) mod 1.;
    variable r = abs(z0);
    variable l = 0.5*r;
    if(r>1)  l = 1.-0.5/r;
    RGB[i] = hsl2rgb(h, s, l);
  }

  if(arrayshape==NULL)
    return RGB[0];
  else
  { reshape(z, arrayshape);
    return _reshape(RGB, arrayshape);
  }
}
