define mix_rgb_colors(rgb1,rgb2,fraction) {
%!%+
%\function{mix_rgb_colors}
%\synopsis{mix two rgb colors}
%\usage{rgb=mix_rgb_colors(rgb1,rgb2,fraction)}
%\description
% This function mixes two colors in rgb space. Given the rgb values in the
% usual encoding in one 24bit integer, the color is mixed according to 
% new rgb=rgb1*fraction+rgb2*(1-fraction)
% The operations are analoguous to the color mixing performed by the xcolor
% package of LaTeX (the operation is similar to LaTeX's color1!fraction!color2
% syntax). 
% The function returns the integer specifying the new rgb values.
%\seealso{xfig_mix_color}
%!%-


  % get rgb values 
  variable bb1=double(rgb1 & 0x0000FF);
  variable gg1=double((rgb1>>8) & 0x0000FF);
  variable rr1=double((rgb1>>16) & 0x0000FF);
  
  variable bb2=double(rgb2 & 0x0000FF);
  variable gg2=double((rgb2>>8) & 0x0000FF);
  variable rr2=double((rgb2>>16) & 0x0000FF);

  if (fraction<0.) {fraction=0.;}
  if (fraction>1.) {fraction=1.;}

  % Mix colors in floating point space
  variable rr=rr1 * fraction + (1.-fraction)*rr2;
  variable gg=gg1 * fraction + (1.-fraction)*gg2;
  variable bb=bb1 * fraction + (1.-fraction)*bb2;
  if (rr>255.) { rr=255;};
  if (gg>255.) { gg=255;};
  if (bb>255.) { bb=255;};

  if (rr<0) { rr=0;};
  if (gg<0) { gg=0;};
  if (bb<0) { bb=0;};

  % put everything back together
  variable rgb=(int(rr) << 16) | (int(gg) << 8) | int(bb);

  return rgb;
}
