define rgb2hex()
%!%+
%\function{rgb2hex}
%\synopsis{converts r, g, b values between 0 and 1 to a 24-bit integer}
%\usage{Integer_Type rgb2hex(Double_Type r, g, b)}
%\qualifiers{
%\qualifier{str}{return value is a string with preceding "#".}
%\qualifier{des}{input color will be desaturated to gray-scale values.
%           If a value is given, the saturation will be scaled accordingly.}
%}
%\seealso{rgb2hsl}
%!%-
{
  variable r, g, b;
  switch(_NARGS)
  { case 3: (r, g, b) = (); }
  { return help(_function_name()); }

  ifnot(length(r) == length(g) ==length(b))
    return "ERROR (rgb2hex): color arrays must of be of the same length.";
  
  variable col = (int(_max(0, _min(r*256, 255)))<<16)
		+(int(_max(0, _min(g*256, 255)))<< 8)
		+ int(_max(0, _min(b*256, 255)))    ;

  if (qualifier_exists("des")) {
    variable h,s,l ;
    (h, s, l) = rgb2hsl(col);
    variable s_scale = qualifier("des");
    if(s_scale == NULL)  s_scale = 0;
    s *= s_scale;
    col = hsl2rgb(h, _max(0, _min(s, 1)), l);
  }

  if (qualifier_exists("str") ) {
  col = array_map (String_Type, &sprintf, "#%06X", col) ;
  }
  
  return length(col)==1 ? col[0] : col;
}
