%%%%%%%%%%%%%%
define rgb2hsl()
%%%%%%%%%%%%%%
%!%+
%\function{rgb2hsl}
%\synopsis{converts (red, green, blue) values to (hue, saturation, lightness)}
%\usage{(Double_Type h, s, l) = rgb2hsl(Integer_Type rgb);
%\altusage{(Double_Type h, s, l) = rgb2hsl(Double_Type r, g, b);}
%}
%\description
%    In the first usage, rgb is an (array of) 24 bit RGB values.
%    In the second form, r, g, and b are values between 0. and 1.
%
%    The return values are always normalized to [0, 1].
%    See \code{hsl2rgb} for a definition of hue, saturation, lightness.
%\seealso{hsl2rgb}
%!%-
{
  variable r, g, b, h, s, l;
  switch(_NARGS)
  { case 1:
      variable rgb = ();
      r = ((rgb & 0xFF0000) >> 16)/255.;
      g = ((rgb & 0x00FF00) >>  8)/255.;
      b = ( rgb & 0x0000FF       )/255.;
  }
  { case 3: (r, g, b) = (); }
  { return help(_function_name()); }

  if(typeof(r)==Array_Type)
  {
    variable dim = array_shape(r);
    variable i, n = length(r);
    reshape(r, n);
    reshape(g, n);
    reshape(b, n);
    h = Double_Type[n];
    s = Double_Type[n];
    l = Double_Type[n];
    _for i (0, n-1, 1)
      (h[i], s[i], l[i]) = rgb2hsl(r[i], g[i], b[i]);
    reshape(r, dim);
    reshape(g, dim);
    reshape(b, dim);
    reshape(h, dim);
    reshape(s, dim);
    reshape(l, dim);
    return h, s, l;
  }

  variable mn, mx; (mn, mx) = min_max([r, g, b]);
  l = (mx+mn)/2.;
  if(mx == mn)
  {
    h = 0;
    s = 0;
  }
  else
  {
    variable tmp = 1./6./(mx-mn);
    if(r == mx)   h = ((g-b)*tmp + 1.) mod 1.;
    if(g == mx)   h =  (b-r)*tmp + 1./3.;
    if(b == mx)   h =  (r-g)*tmp + 2./3.;
    s = (l<=.5) ? (mx-mn)/(mx+mn) : (mx-mn)/(2-mx-mn);
  }

  return h, s, l;
}
