define rgb2r_g_b( rgb )
%!%+
%\function{rgb2r_g_b}
%\synopsis{converts a ( rgb ) color to (red, green, blue)}
%\usage{Integer_Type r,g,b = rgb2r_g_b( Integer_Type[] );}
%\description
%    Converts a color encoded in one Integer into its single
%    r, g, b values.
%    The return value is a(n array of) 24 bit RGB value(s)
%    in Integer_Type interval [0,255].
%\qualifiers{
%\qualifier{norm}{return values are in Double_Type interval [0,1].}
%\qualifier{str}{If qualifier is given ('str'=NULL) r,g,b will be retruned
%                  as strings. If 'str' != NULL, e.g., 'str'=',', than
%                  the singe color channels are combined and returned:
%                  r + str + g + str + b}
%}
%
%\seealso{rgb2hsl, hex2rgb}
%!%-
{
  
  variable r = (rgb & 0xFF0000) >> 16;
  variable g = (rgb & 0xFF00) >> 8;
  variable b = rgb & 0xFF;

  if( qualifier_exists("norm") ){
    r /= 1.*255;
    g /= 1.*255;
    b /= 1.*255;
  }
  variable str = qualifier("str");
  if( qualifier_exists("str") ){
    r = array_map( String_Type, &string, r );
    g = array_map( String_Type, &string, g );
    b = array_map( String_Type, &string, b );
    if( str != NULL ){
      return r+str+g+str+b;
    }
  }
  
  return r,g,b;
}
