%%%%%%%%%%%%%%%%%%%
define MJD2UNIXtime()
%%%%%%%%%%%%%%%%%%%
%!%+
%\function{MJD2UNIXtime}
%\synopsis{converts modified Julian date to UNIX time}
%\usage{Long_Type MJD2UNIXtime(Double_Type MJD)}
%\description
%    Time formats and conversion functions:
%      Long_Type secs = MJD2UNIXtime(Double_Type MJD);
%      Double_Type MJD = UNIXtime2MJD(Long_Type secs);
%
%      Struct_Type tm = localtime(Long_Type secs);  % or gmtime
%      Long_Type secs = mktime(Struct_Type tm);
%
%      String_Type str = strftime(String_Type fmt, Struct_Type tm);
%
%    Current time and date:
%      Long_Type secs = _time();
%      String_Type str = time();
%\seealso{UNIXtime2MJD, localtime, gmtime, mktime}
%!%-
{
  variable MJD;
  switch(_NARGS)
  { case 1: MJD = (); }
  { help(_function_name()); return; }

  % note: we need to first typecast MJD to a 64bit floating point
  % number (Double_Type == Float64_Type) in order to have the
  % calculation performed in 64bit. If, e.g., MJD is given of
  % Integer_Type == Int32_Type, then some dates might exceed
  % INT32_MAX because the calculation would be performed in 32bit.
  % Finally, because UNIXtime is an integer number, we need to
  % convert the result to an integer using 64bit, i.e., typecast
  % to Long_Type == Int64_Type
  return typecast((typecast(MJD, Float64_Type)-40587)*86400, Int64_Type);
}
