define MJDofDate()
%!%+
%\function{MJDofDate}
%\synopsis{calculates the modified Julian date}
%\usage{Double_Type MJD = MJDofDate( struct { year, month, day, hour, minute, second } );
%\altusage{Double_Type MJD = MJDofDate(year, month, day[, hour[, minute[, second]]]);}
%}
%\description
%    The Gregorian or Julian date is either given by a structure or by at
%    least 3 integer arguments. Fractional parts are ignored. 
%    The default values for hour, minute and second are 0, fractional
%    hours etc. are properly taken into account.
%
%    This function is array safe and equivalent to calling JDofDate with the
%    mjd qualifier.
%\qualifiers{
%\qualifier{julian_calendar}{return the JD/MJD for the date in the Julian calendar rather
%       the Gegorian date. This does not make much sense for the MJD, which was negative
%       for the time when the Julian calendar was used.}
%}
%\seealso{JDofDate,dateOfMJD}
%!%-
{
  variable year, month, day, hour=0, minute=0, second=0;
  switch(_NARGS)
  { case 1:
      variable date = ();
      if(typeof(date)==Struct_Type)
      { year   = date.year;
        month  = date.month;
        day    = date.day;
        hour   = date.hour;
        minute = date.minute;
        second = date.second;
      }
      else
      { help(_function_name());
        return;
      }
  }
  { case 3: (year, month, day) = (); }
  { case 4: (year, month, day, hour) = (); }
  { case 5: (year, month, day, hour, minute) = (); }
  { case 6: (year, month, day, hour, minute, second) = (); }
  { help(_function_name()); return; }
  if (qualifier_exists("julian_calendar")) {
      return JDofDate(year,month,day,hour,minute,second;mjd,julian_calendar);
  } 
  return JDofDate(year,month,day,hour,minute,second;mjd);
}
