define MJDofDateString ()
%!%+
%\function{MJDofDateString}                                                                                         
%\synopsis{calculates the MJD from a string including year, month, day [, hour [, minute [, second]]]}
%\usage{Double_Type \code{mjd} = MJDofDateString(String_Type \code{date_string});}
%\qualifiers{
%\qualifier{verbose}{prints the found values for YYYY-MM-DD hh:mm:ss.sss}
%\qualifier{no_2digit_correction}{see `parseDateString`}
%}
%\description
%    This function calculates the Modified Julian Date from a String
%    using the `parseDateString` function.
%\example
%    MJDofDateString("69/07/21");
%    MJDofDateString("1969-07-21 02:56:30.44");
%    MJDofDateString("sdf1969--X07..21QQ_02/56((30,,44xyz");
%\seealso{parseDateString}
%!%-
{
  variable str;
  switch(_NARGS)
    { case 1: str = (); }
    { help(_function_name()); return;}

  % parse string
  variable tm = parseDateString(str);

  % correct month and year
  tm.tm_mon += 1;
  tm.tm_year += 1900;
  
  if (qualifier_exists("verbose")) {
    vmessage("%s uses YYYY-MM-DD: %04d-%02d-%02d  %02d:%02d:%f",
      _function_name(), tm.tm_year, tm.tm_mon, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec
    );
  }
  return MJDofDate(tm.tm_year, tm.tm_mon, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);
}
