%%%%%%%%%%%%%%%%%%%
define UTC2UNIXtime()
%%%%%%%%%%%%%%%%%%%
%!%+
%\function{UTC2UNIXtime}
%\synopsis{converts Universal Time (GMT) to seconds since 1970-01-01, 0:00 UTC}
%\usage{Long_Type secs = UTC2UNIXtime(Struct_Type tm);
%\altusage{Long_Type secs = UTC2UNIXtime(Integer_Type Y[, m[, d[, H[, M[, S]]]]]);}
%}
%\description
%    \code{UTC2UNIXtime} can be seen as the inverse of the \code{gmtime} function,
%    operating on Coordinated Universal Time instead of dates in a local time zone,
%    as \code{mktime} does, which is the inverse of \code{localtime}.
%
%    \code{UTC2UNIXtime} can also take the components of the date \code{Y}-\code{m}-\code{d}, \code{H}:\code{M}:\code{S} UTC
%    as separate arguments. The year \code{Y} is then given directly (unlike \code{tm.tm_year = Y - 1900}),
%    and likewise the month \code{m} (unlike \code{tm.tm_mon = m-1}).
%    For omitted arguments, \code{m=1}, \code{d=1}, \code{H=0}, \code{M=0}, and \code{S=0} are assumed.
%\seealso{mktime, gmtime, localtime}
%!%-
{
  variable tm=NULL, Y, m=1, d=1, H=0, M=0, S=0;
  switch(_NARGS)
  { case 1:  Y = ();  if(typeof(Y)==Struct_Type)  tm = Y; }
  { case 2: (Y, m            ) = (); }
  { case 3: (Y, m, d         ) = (); }
  { case 4: (Y, m, d, H      ) = (); }
  { case 5: (Y, m, d, H, M   ) = (); }
  { case 6: (Y, m, d, H, M, S) = (); }
  { help(_function_name()); return; }

  if(tm==NULL) tm = struct { tm_sec=S, tm_min=M, tm_hour=H, tm_mday=d, tm_mon=m-1, tm_year=Y-1900, tm_wday=0, tm_yday=0, tm_isdst=0 };
  variable tm_test = gmtime(mktime(struct { tm_sec=0, tm_min=0, tm_hour=12, tm_mday=2, tm_mon=0, tm_year=100, tm_wday=0, tm_yday=0, tm_isdst=0 }));
  variable deltaH = 12-tm_test.tm_hour + 24*(2-tm_test.tm_mday);
  return mktime(tm) + 3600*deltaH;
}
