define dayOfWeek()
%!%+
%\function{dayOfWeek}
%\synopsis{returns the day of the week of the given date}
%\usage{Integer_Type dayOfWeek(Integer_Type year, month, day);}
%\description
%    This function returns the day of the week from 1 (Monday) to
%    7 (Sunday) of the given date in the Gregorian (default) or the
%    Julian calendar.
%
%\qualifiers{
%\qualifier{julian_calendar}{: the date is in the Julian calendar
%                              (default: Gregorian calendar)}
%}
%\example
%    dayOfWeek(1837, 9, 9);
%!%-
{
  variable y, m, d;
  switch(_NARGS)
    { case 3: (y,m,d) = (); }
    { help(_function_name()); return;}

  % sanity checks
  if (typeof(y) != Integer_Type) { vmessage("error (%s): year must be an integer", _function_name); return; }
  if (typeof(m) != Integer_Type) { vmessage("error (%s): month must be an integer", _function_name); return; }
  if (typeof(d) != Integer_Type) { vmessage("error (%s): day must be an integer", _function_name); return; }

  %jw do NOT check on month or day validity, since
  %astronomers sometimes use the 0th or the -1st of the
  %day and similar weird things...
  
  % return day of week
  % NB calculating the JD for 13h avoids potential
  % round off
  if (qualifier_exists("julian_calendar")) {
      return 1 + (int(JDofDate(y,m,d,13,0,0;julian_calendar)) mod 7);
  }
  
  return 1 + (int(JDofDate(y,m,d,13,0,0)) mod 7);
}
