define jd2year()
%!%+
%\function{jd2year}
%\synopsis{transform a JD into a fractional year}
%\usage{Double_Type year = jd2year(Double_Type JD);}
%\qualifiers{
%\qualifier{mjd}{The argument is given in MJD, not in JD.}
%}
%\description
%  NOTE: This routine takes into account leap years. This is good for
%  plotting, but this means that there are slight phase shifts that could be
%  problems when doing time series analysis. For such purposes best use
%  EpochofJD!
%\seealso{EpochofJD}
%!%-
{
  variable JD;
  switch(_NARGS)
  { case 1: JD = (); }
  { help(_function_name()); return; }

  if (qualifier_exists("mjd")) {
      JD=mjd2jd(JD);
  }
  
  if(typeof(JD)==Array_Type) {
      return array_map(Double_Type, &jd2year, JD);
  }
  
  variable date=DateOfJD(JD);

  variable jd0=JDofDate(date.year  ,1,1,0,0,0.);
  variable jd1=JDofDate(date.year+1,1,1,0,0,0.);

  return date.year+(JD-jd0)/(jd1-jd0);

}
