define erfinv(z)
%!%+
%\function{erfinv}
%\synopsis{Computes the inverse error function in abs(z)<1}
%\usage{Double_Type erfinv(Double_Type z);}
%\description
%    Can be used to calculate a confidence level by
%    \code{sqrt(2)*erfinv(fraction)}
%
%    The function is implemented according to Sergei Winitzki's 2008\n
%    publication: "A handy approximation for the error function and its\n
%    inverse". The approximation has a relative error <1.3E-4.
%\seealso{cerf,cerfc}
%!%-
{
   if (max(z)>=1. || min(z)<=-1.) { % z can be array
      vmessage("Valid range only abs(z)<1");
      return 0;
   }
   variable a = 0.147;
   return sign(z)*sqrt( sqrt( (2/(PI*a)+log(1-z^2)/2)^2-log(1-z^2)/a ) - (2/(PI*a)+log(1-z^2)/2) );
}
